-- ------------------------------------------------------------------------------
-- rp:Friends
-- by Oraibi, Moon Guard (US) server
--
-- Ora's twitter: http://twitter.com/oraibimoonguard
--
-- ------------------------------------------------------------------------------
--
-- This work is licensed under the Creative Commons Attribution 4.0 International
-- (CC BY 4.0) license. To view a copy of the license, visit
--
--     https://creativecommons.org/licenses/by/4.0/
--
-- This license is acceptable for Free Cultural Works.
--
-- You are free to:
--
-- Share -- copy and redistribute the material in any medium or format
-- Adapt -- remix, transform, and build upon the material
--         for any purpose, even commercially.
-- 
-- The licensor cannot revoke these freedoms as long as you follow the 
-- license terms.
-- 
-- Under the following terms:
-- Attribution -- You must give appropriate credit, provide a link to the
--         license, and indicate if changes were made. You may do
--         so in any reasonable manner, but not in any way that
--         suggests the licensor endorses you or your use.
--
-- No additional restrictions -- You may not apply legal terms or
--         technological measures that legally restrict others from doing
--         anything the license permits.
--
-- Notices:
--
-- You do not have to comply with the license for elements of the material
-- in the public domain or where your use is permitted by an applicable
-- exception or limitation.
--
-- No warranties are given. The license may not give you all of the
-- permissions necessary for your intended use. For example, other rights
-- such as publicity, privacy, or moral rights may limit how you use the
-- material.
-- ------------------------------------------------------------------------------

-- Init: program initalization

local addOnName, RP_FRIENDS = ...;
RP_FRIENDS.addOnName = addOnName;

for _, item in ipairs({ "cache", "const", "keybinds", "locale", "startup", "utils", }) do RP_FRIENDS[item] = {} end;

local startupFrame = CreateFrame("frame")
startupFrame:RegisterEvent("ADDON_LOADED");
startupFrame:SetScript("OnEvent", 
  function(self, event, addOnLoaded)
    if addOnLoaded == addOnName
       then -- make sure our saved variables exist
            if not RP_FriendsDB          then RP_FriendsDB          = {} end;
            if not RP_FriendsDB.settings then RP_FriendsDB.settings = {} end;
            local state      = {};

            if _G["msp"]      then state.msp  = true end; -- yay we don't need to use our own, we can piggyback off someone else's!
            if _G["TRP3_API"] then state.trp3 = true end;
            if _G["mrp"]      then state.mrp  = true end;

            RP_FRIENDS.cache.startLog = { "This table contains a list of each startup routine excuted, in the order in which they were executed." };
            for _, startupRoutine in ipairs(RP_FRIENDS.startup)
            do  local started = startupRoutine(state)
                table.insert(RP_FRIENDS.cache.startLog, started)
                if started:match("%-break$") and state.err then break end; -- we have enough to give error messages
            end; -- for

            RP_FRIENDS.startup = nil; -- save memory

            table.insert(RP_FRIENDS.cache.startLog, "Startup routines finished.");

            -- if the user wants the login message, show it
            if RP_FRIENDS.utils.config.get("LOGIN_MESSAGE")   then RP_FRIENDS.utils.text.notify(RP_FRIENDS.utils.text.version(state)); end; -- if
            if RP_FRIENDS.utils.config.get("CHANGES_MESSAGE") and RP_FRIENDS.utils.config.get("LOGIN_MESSAGE") 
            then RP_FRIENDS.utils.text.notify(RP_FRIENDS.utils.text.changes(state)); end; -- if

            RP_FRIENDS.cache.state    = state;


       end; -- if addOnLoaded
    end); -- function
_G.RP_FRIENDS = RP_FRIENDS;
